/* DS18B20 temp reading and LCD display example

Version 1.0.2
by John Dimo


This program reads the temperature from a DS18B20 and displays
it in Celsius and Fahrenheit with 2 decimal point accuracy.
Designed for standard temp sensor with 3 wires. Not the two wire
parasitic version.

1.0.2 - Code cleanup minor changes to comments
1.0.1 - Fixed issue with Fahrenheit fraction not displaying correctly
1.0 initial version
*/

#define TEMP_PIN  7 //Setup the digital pin we will use
#include <LiquidCrystal.h>

LiquidCrystal lcd(12, 11, 5, 4, 3, 2); //Setup pins for LCD Display
byte newChar[8] = { // create degree symbol for lcd
	B01100,	
        B10010, 
        B10010,
	B01100,	
        B00000,	
        B00000,
	B00000,	
        B00000 };

void setup() {
    digitalWrite(TEMP_PIN, LOW);
    pinMode(TEMP_PIN, INPUT);      // sets the digital pin as input
    lcd.begin(16, 2); // initilize lcd as a 16 char, 2 line screen
    for(int i=0; i<2; i++){
      lcd.setCursor(0,i);
      lcd.print("Temp:");
    }
    lcd.createChar(0, newChar); // assign our new character with value 0
}

void loop(){
  int SignBit, Whole, Fract; //setup variables
  int HighByte, LowByte, TReading; // setup variables
  long Celsius; //Celsius Variable
  long Fahrenheit; // fahrenheit Variable
  OneWireReset(TEMP_PIN);
  OneWireOutByte(TEMP_PIN, 0xcc);
  OneWireOutByte(TEMP_PIN, 0x44); // perform temperature conversion
  OneWireReset(TEMP_PIN);
  OneWireOutByte(TEMP_PIN, 0xcc);
  OneWireOutByte(TEMP_PIN, 0xbe);

  LowByte = OneWireInByte(TEMP_PIN);
  HighByte = OneWireInByte(TEMP_PIN);
  TReading = (HighByte << 8) + LowByte;
  SignBit = TReading & 0x8000;  // test most sig bit
  if (SignBit) // negative
  {
    TReading = (TReading ^ 0xffff) + 1; // 2's comp
  }
  
  //Genereate Celsius Temperature Reading and Display on LCD
  Celsius = (6 * TReading) + TReading / 4;    // multiply by (100 * 0.0625) or 6.25
  Whole = Celsius / 100;  // separate off the whole and fractional portions
  Fract = Celsius % 100;
  lcd.setCursor(6,0);
  displaytemp(SignBit, Whole, Fract, 'C'); //display C temp
  
  // Generate Fahrenheit by converting the celsius reading and display on LCD
  if (!SignBit)
  {
    Fahrenheit = Celsius * 1.8 + 3200; //both are positive, convert to farhenheit
  }
  
  if (SignBit && Celsius <= 1777) //The point at which C goes negative and F is still positive
  {
    Fahrenheit = Celsius * 1.8;
    Fahrenheit = 3200 - Fahrenheit;
    SignBit = 0;
  }
  
  if (SignBit && Celsius > 1777) //The point where both are negative
  {
    Fahrenheit = Celsius - 1777;
    Fahrenheit = Fahrenheit * 1.8;
  }
  
  Whole = Fahrenheit / 100;  // separate off the whole and fractional portions
  Fract = Fahrenheit % 100;
  lcd.setCursor(6,1);
  displaytemp(SignBit, Whole, Fract,'F'); // display F temp
  delay(1000);      // 1 second delay between temp readings
}

void OneWireReset(int Pin) // reset temp sensor
{
     digitalWrite(Pin, LOW);
     pinMode(Pin, OUTPUT); // bring low for 500 us
     delayMicroseconds(500);
     pinMode(Pin, INPUT);
     delayMicroseconds(500);
}

void OneWireOutByte(int Pin, byte d) // output byte d (least sig bit first).
{
   byte n;

   for(n=8; n!=0; n--)
   {
      if ((d & 0x01) == 1)  // test least sig bit
      {
         digitalWrite(Pin, LOW);
         pinMode(Pin, OUTPUT);
         delayMicroseconds(5);
         pinMode(Pin, INPUT);
         delayMicroseconds(60);
      }
      else
      {
         digitalWrite(Pin, LOW);
         pinMode(Pin, OUTPUT);
         delayMicroseconds(60);
         pinMode(Pin, INPUT);
      }

      d=d>>1; // now the next bit is in the least sig bit position.
   }

}

byte OneWireInByte(int Pin) // read byte, least sig byte first
{
    byte d, n, b;

    for (n=0; n<8; n++)
    {
        digitalWrite(Pin, LOW);
        pinMode(Pin, OUTPUT);
        delayMicroseconds(5);
        pinMode(Pin, INPUT);
        delayMicroseconds(5);
        b = digitalRead(Pin);
        delayMicroseconds(50);
        d = (d >> 1) | (b<<7); // shift d to right and insert b in most sig bit position
    }
    return(d);
}

void displaytemp(int SignBit, int Whole, int Fract, char temptype) {
  if (SignBit) // If its negative
  {
    lcd.print("-");
  }
  lcd.print(Whole);
  lcd.print(".");
  if (Fract < 10)
  {
    lcd.print("0");
  }
  lcd.print(Fract);
  lcd.write(0); // display our custom character
  
  lcd.print(temptype);
  lcd.print("  ");
}
